
/************************************************************************/
/*  Startup for GNU toolchain                                           */
/*  Version     V1.0                                                    */
/*  Date        2014-05-22                                             */
/*  Target-mcu  SE6E1A1                                                 */
/************************************************************************/


/*
// <h> Stack Configuration
//   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
// </h>
*/

	.equ    Stack_Size, 0x00000100
    .section ".stack", "w"
    .align  3
    .globl  __FM0_stack_mem
    .globl  __FM0_stack_size
__FM0_stack_mem:
    .if     Stack_Size
    .space  Stack_Size
    .endif
    .size   __FM0_stack_mem,  . - __FM0_stack_mem
    .set    __FM0_stack_size, . - __FM0_stack_mem


/*
// <h> Heap Configuration
//   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
// </h>
*/

    .equ    Heap_Size,  0x00000000
    
    .section ".heap", "w"
    .align  3
    .globl  __FM0_heap_start
    .globl  __FM0_heap_end
__FM0_heap_start:
    .if     Heap_Size
    .space  Heap_Size
    .endif
__FM0_heap_end:


/* Vector Table */

    .section ".FM0.interrupt_vector"
    .globl  __FM0_interrupt_vector_cortex_m
    .type   __FM0_interrupt_vector_cortex_m, %object

__FM0_interrupt_vector_cortex_m:
    .long   __FM0_stack                 /* Top of Stack                 */
    .long   __FM0_reset                 /* Reset Handler                */
    .long   NMI_Handler                 /* NMI Handler                  */
    .long   HardFault_Handler           /* Hard Fault Handler           */
    .long   0				            /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   SVC_Handler                 /* SVCall Handler               */
    .long   0                           /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   PendSV_Handler              /* PendSV Handler               */
    .long   SysTick_Handler             /* SysTick Handler              */

    /* External Interrupts */

	.long     CSV_IRQHandler            /*0: Clock Super Visor 			*/
	.long     SWDT_IRQHandler           /*1: Software Watchdog Timer	*/
	.long     LVD_IRQHandler            /*2: Low Voltage Detector		*/
	.long     MFT_WFG_DTIF_IRQHandler   /*3: Wave Form Generator / DTIF	*/
	.long     EXINT0_7_IRQHandler       /*4: External Interrupt Request ch.0 to ch.7 */
	.long     DummyHandler_1            /*5: Reserved					*/
	.long     DT_QDU_IRQHandler         /*6: Dual Timer / Quad Decoder	*/
	.long     MFS0_RX_IRQHandler        /*7: MultiFunction Serial ch.0 	*/
	.long     MFS0_TX_IRQHandler        /*8: MultiFunction Serial ch.0 	*/
	.long     MFS1_RX_IRQHandler        /*9: MultiFunction Serial ch.1	*/
	.long     MFS1_TX_IRQHandler        /*10: MultiFunction Serial ch.1	*/
	.long     DummyHandler_2            /*11: Reserved                  */
	.long     DummyHandler_3            /*12: Reserved                  */
	.long     MFS3_RX_IRQHandler        /*13: MultiFunction Serial ch.3	*/
	.long     MFS3_TX_IRQHandler        /*14: MultiFunction Serial ch.3 */
	.long     DummyHandler_4            /*15: Reserved					*/
	.long     DummyHandler_5            /*16: Reserved					*/
	.long     DummyHandler_6            /*17: Reserved					*/
	.long     DummyHandler_7            /*18: Reserved					*/
	.long     DMAC0_IRQHandler          /*19: DMA ch.0					*/
	.long     DMAC1_IRQHandler          /*20: DMA ch.1					*/
	.long     DummyHandler_8            /*21: Reserved					*/
	.long     DummyHandler_9            /*22: Reserved					*/
	.long     PPG_IRQHandler            /*23: PPG						*/
	.long     TIM_IRQHandler            /*24: OSC / PLL / Watch Counter / RTC */
	.long     ADC0_IRQHandler           /*25: ADC0						*/
	.long     DummyHandler_10           /*26: Reserved					*/
	.long     DummyHandler_11           /*27: Reserved					*/
	.long     MFT_FRT_IRQHandler        /*28: Free-run Timer			*/
	.long     MFT_IPC_IRQHandler        /*29: Input Capture				*/
	.long     MFT_OPC_IRQHandler        /*30: Output Compare			*/
	.long     BT0_3_FLASH_IRQHandler    /*31: Base Timer ch.0 to ch.3 / Flash */
	
    .size   __FM0_interrupt_vector_cortex_m, . - __FM0_interrupt_vector_cortex_m


    .thumb


/* Reset Handler */

    .section .FM4.reset,"x",%progbits
    .thumb_func
    .globl  __FM0_reset_cortex_m
    .type   __FM0_reset_cortex_m, %function
__FM0_reset_cortex_m:
    .fnstart
.ifdef Debug_RAM
/* this routine is used for .bss sections initialization by RAM debugging "Debug_RAM = 1" */
/* Clear .bss section (Zero init) */
	MOV     R0, #0
	LDR     R1, =_sbss
	LDR     R2, =_ebss
	CMP     R1,R2
	BEQ     start_main
Loop_bss:
	CMP     R1, R2
	BHS		start_main
	STR   	R0, [R1]
	ADD		R1, #4
	BLO     Loop_bss

.else
/* this routine is used for .data and .bss sections initialization by ROM debugging "Debug_RAM = 0" */
init_data:
	LDR		R0, =_sdata
	LDR		R1, =_etext
	LDR		R2, =data_size
	MOV		R3, #0
loop_init_data:
	CMP		R3, R2
	BCS		zero_bss
    LDR		R4,[R1,#0]
	STR		R4,[R0,#0]
	ADD		R3, #4
	ADD		R1, #4
	ADD		R0, #4
	B		loop_init_data

/* Clear .bss section (Zero init) */

zero_bss:
	LDR		R0, =_sbss
	LDR		R2, =bss_size
	MOV		R3, #0
	MOV		R4, #0
loop_zero_bss:
	CMP		R3, R2
	BCS		start_main
	STR		R4,[R0,#0]
	ADD		R3, #4
	ADD		R0, #4
	B		loop_zero_bss
.endif
start_main:
	LDR     R0, =SystemInit
    BLX     R0
    LDR     R0,=main
    BX      R0

    .pool
    .cantunwind
    .fnend
    .size   __FM0_reset_cortex_m,.-__FM0_reset_cortex_m

    .section ".text"

/* Exception Handlers */

    .weak   NMI_Handler
    .type   NMI_Handler, %function
NMI_Handler:
    B       .
    .size   NMI_Handler, . - NMI_Handler

    .weak   HardFault_Handler
    .type   HardFault_Handler, %function
HardFault_Handler:
    B       .
    .size   HardFault_Handler, . - HardFault_Handler

    .weak   SVC_Handler
    .type   SVC_Handler, %function
SVC_Handler:
    B       .
    .size   SVC_Handler, . - SVC_Handler

    .weak   PendSV_Handler
    .type   PendSV_Handler, %function
PendSV_Handler:
    B       .
    .size   PendSV_Handler, . - PendSV_Handler

    .weak   SysTick_Handler
    .type   SysTick_Handler, %function
SysTick_Handler:
    B       .
    .size   SysTick_Handler, . - SysTick_Handler


/* IRQ Handlers */

    .globl  Default_Handler
    .type   Default_Handler, %function
Default_Handler:
    B       .
    .size   Default_Handler, . - Default_Handler

    .macro  IRQ handler
    .weak   \handler
    .set    \handler, Default_Handler
    .endm
	
	IRQ     CSV_IRQHandler           
	IRQ     SWDT_IRQHandler          
	IRQ     LVD_IRQHandler           
	IRQ     MFT_WFG_DTIF_IRQHandler  
	IRQ     EXINT0_7_IRQHandler      
	IRQ     DummyHandler_1          
	IRQ     DT_QDU_IRQHandler        
	IRQ     MFS0_RX_IRQHandler       
	IRQ     MFS0_TX_IRQHandler        
	IRQ     MFS1_RX_IRQHandler       
	IRQ     MFS1_TX_IRQHandler       
	IRQ     DummyHandler_2          
	IRQ     DummyHandler_3          
	IRQ     MFS3_RX_IRQHandler       
	IRQ     MFS3_TX_IRQHandler       
	IRQ     DummyHandler_4          
	IRQ     DummyHandler_5          
	IRQ     DummyHandler_6          
	IRQ     DummyHandler_7          
	IRQ     DMAC0_IRQHandler         
	IRQ     DMAC1_IRQHandler         
	IRQ     DummyHandler_8          
	IRQ     DummyHandler_9          
	IRQ     PPG_IRQHandler           
	IRQ     TIM_IRQHandler           
	IRQ     ADC0_IRQHandler          
	IRQ     DummyHandler_10          
	IRQ     DummyHandler_11          
	IRQ     MFT_FRT_IRQHandler       
	IRQ     MFT_IPC_IRQHandler       
	IRQ     MFT_OPC_IRQHandler       
	IRQ     BT0_3_FLASH_IRQHandler   
	
    .end
